﻿using System;
using Xamarin.Forms;
using Xamarin.Forms.Xaml;
using Plugin.BxlMpXamarinSDK;
using Plugin.BxlMpXamarinSDK.Abstractions;
using System.Threading.Tasks;
using System.Threading;

namespace XamarinMpSDKDemo
{
    [XamlCompilation(XamlCompilationOptions.Compile)]
    public partial class LineModePage : ContentPage
    {
        private MPosControllerPrinter _printer;
        private MposConnectionInformation _connectionInfo;
        private static SemaphoreSlim _printSemaphore = new SemaphoreSlim(1, 1);

        public LineModePage()
        {
            InitializeComponent();
            _printer        = null;
            _connectionInfo = null;
        }
        protected override void OnDisappearing()
        {
            if (_printer != null){
                OnDeviceCloseClicked(this, null);
            }
        }
        protected override void OnBindingContextChanged()
        {
            if (BindingContext == null)
                return;

            var connectionInfo = BindingContext as MposConnectionInformation;

            if (connectionInfo != null)
            {
                _connectionInfo = connectionInfo;

                switch (_connectionInfo.IntefaceType)
                {
                    case MPosInterfaceType.MPOS_INTERFACE_BLUETOOTH:
                    case MPosInterfaceType.MPOS_INTERFACE_WIFI:
                    case MPosInterfaceType.MPOS_INTERFACE_ETHERNET:
                        lblAddress.Text = connectionInfo.Address;
                        btnOpenService.IsEnabled = true;
                        break;
                    default:
                        return;
                }
            }
        }

        void PrinterStatusEvent(uint status)
        {
            // Any view interaction or any UI interaction should always be done in the main thread.
            Device.BeginInvokeOnMainThread(() =>
            {
                lblStatus.Text = "";

                if (status == (uint)MPosPrinterStatus.MPOS_STATUS_NORMAL){
                    lblStatus.Text = "NORMAL ";
                    return;
                }
                if ((status & (uint)MPosPrinterStatus.MPOS_STATUS_PRINTER_ERROR) != 0)
                    lblStatus.Text += " PRINTER ERROR ";
                if ((status & (uint)MPosPrinterStatus.MPOS_STATUS_PRINTER_PAPER_NEAREND) != 0)
                    lblStatus.Text += " PAPER NEAR END ";
                if ((status & (uint)MPosPrinterStatus.MPOS_STATUS_PRINTER_COVEROPEN) != 0)
                    lblStatus.Text += " COVER OPEN ";
                if ((status & (uint)MPosPrinterStatus.MPOS_STATUS_PRINTER_PAPEREMPTY) != 0)
                    lblStatus.Text += " PAPER EMPTY ";
            });
        }

        async Task<MPosControllerPrinter> OpenPrinterService(MposConnectionInformation connectionInfo)
        {
            if (connectionInfo == null)
                return null;

            if (_printer != null)
                return _printer;

            btnOpenService.IsEnabled = false;

            _printer = MPosDeviceFactory.Current.createDevice(MPosDeviceType.MPOS_DEVICE_PRINTER) as MPosControllerPrinter;

            switch (connectionInfo.IntefaceType)
            {
                case MPosInterfaceType.MPOS_INTERFACE_BLUETOOTH:
                case MPosInterfaceType.MPOS_INTERFACE_WIFI:
                case MPosInterfaceType.MPOS_INTERFACE_ETHERNET:
                    _printer.selectInterface((int)connectionInfo.IntefaceType, connectionInfo.Address);
                    _printer.selectCommandMode((int)MPosCommandMode.MPOS_COMMAND_MODE_BYPASS);
                    break;
                default:
                    await DisplayAlert("Connection Fail", "Not Supported Interface", "OK");
                    return null;
            }

            await _printSemaphore.WaitAsync();

            try
            {
                var result = await _printer.openService();
                if (result != (int)MPosResult.MPOS_SUCCESS){
                    _printer = null;
                    await DisplayAlert("Connection Fail", "openService failed. (" + result.ToString() + ")", "OK");
                }
            }
            finally{
                _printSemaphore.Release();
            }

            return _printer;
        }

        async void OnDeviceOpenClicked(object sender, EventArgs e)
        {
            if (_printer == null)
            {
                // Prepares to communicate with the printer 
                _printer = await OpenPrinterService(_connectionInfo);

                if(_printer != null){
                    btnPrintText.IsEnabled = true;
                    btnPrint1dCode.IsEnabled = true;
                    btnPrint2dCode.IsEnabled = true;
                    btnPrintImage.IsEnabled = true;
                    btnPrintStatus.IsEnabled = true;
                    btnCloseService.IsEnabled = true;
                    btnOpenService.IsEnabled = false;
                }
                else{
                    btnPrintText.IsEnabled = false;
                    btnPrint1dCode.IsEnabled = false;
                    btnPrint2dCode.IsEnabled = false;
                    btnPrintImage.IsEnabled = false;
                    btnPrintStatus.IsEnabled = false;
                    btnCloseService.IsEnabled = false;
                    btnOpenService.IsEnabled = true;
                }
            }
        }

        async void OnDeviceCloseClicked(object sender, EventArgs e)
        {
            await _printSemaphore.WaitAsync();

            if (_printer != null){
                // If there's nothing to do with the printer, call "closeService" method to disconnect the communication between Host and Printer.
                await _printer.closeService();
                _printer = null;
                btnPrintText.IsEnabled = false;
                btnPrint1dCode.IsEnabled = false;
                btnPrint2dCode.IsEnabled = false;
                btnPrintImage.IsEnabled = false;
                btnPrintStatus.IsEnabled = false;
                btnCloseService.IsEnabled = false;
                btnOpenService.IsEnabled = true;
            }

            _printSemaphore.Release();
        }

        async void OnPrintTextClicked(object sender, EventArgs e)
        {
            // Prepares to communicate with the printer 
            _printer = await OpenPrinterService(_connectionInfo);

            if (_printer == null)
                return;

            try
            {
                await _printSemaphore.WaitAsync();

                uint textCount = 0;
                string printText = "_Hola Xamarin\n";

                // note : Page mode and transaction mode cannot be used together between IN and OUT.
                // When "setTransaction" function called with "MPOS_PRINTER_TRANSACTION_IN", print data are stored in the buffer.
                await _printer.setTransaction((int)MPosTransactionMode.MPOS_PRINTER_TRANSACTION_IN);
                // Printer Setting Initialize
                await _printer.directIO(new byte[] { 0x1b, 0x40 });
                await _printer.printText((textCount++).ToString() + printText);
                await _printer.printText((textCount++).ToString() + printText, new MPosFontAttribute() { CodePage = (int)MPosCodePage.MPOS_CODEPAGE_WPC1252 });
                await _printer.printText((textCount++).ToString() + printText, new MPosFontAttribute() { FontType = MPosFontType.MPOS_FONT_TYPE_B });
                await _printer.printText((textCount++).ToString() + printText, new MPosFontAttribute() { FontType = MPosFontType.MPOS_FONT_TYPE_C });
                await _printer.printText((textCount++).ToString() + printText, new MPosFontAttribute() { Alignment = MPosAlignment.MPOS_ALIGNMENT_CENTER });
                await _printer.printText((textCount++).ToString() + printText, new MPosFontAttribute() { Bold = true });
                await _printer.printText((textCount++).ToString() + printText, new MPosFontAttribute() { Reverse = true });
                await _printer.printText((textCount++).ToString() + printText, new MPosFontAttribute() { Underline = MPosFontUnderline.MPOS_FONT_UNDERLINE_2 });
                await _printer.printText((textCount++).ToString() + printText, new MPosFontAttribute() { Width = MPosFontSizeWidth.MPOS_FONT_SIZE_WIDTH_2});
                // Feed to tear-off position (Manual Cutter Position)
                await _printer.directIO(new byte[] { 0x1b, 0x4a, 0xaf });
            }
            catch (Exception ex){
                await DisplayAlert("Exception", ex.ToString(), "OK");
            }
            finally{
                // Printer starts printing by calling "setTransaction" function with "MPOS_PRINTER_TRANSACTION_OUT"
                await _printer.setTransaction((int)MPosTransactionMode.MPOS_PRINTER_TRANSACTION_OUT);
                // If there's nothing to do with the printer, call "closeService" method to disconnect the communication between Host and Printer.
                _printSemaphore.Release();
            }
        }

        async void OnPrint1dCodeClicked(object sender, EventArgs e)
        {
            // Prepares to communicate with the printer 
            _printer = await OpenPrinterService(_connectionInfo);

            if (_printer == null)
                return;

            try
            {
                await _printSemaphore.WaitAsync();

                // note : Page mode and transaction mode cannot be used together between IN and OUT.
                // When "setTransaction" function called with "MPOS_PRINTER_TRANSACTION_IN", print data are stored in the buffer.
                await _printer.setTransaction((int)MPosTransactionMode.MPOS_PRINTER_TRANSACTION_IN);

                await _printer.directIO(new byte[] { 0x1b, 0x40 });  // Printer Setting Initialize
                await _printer.print1DBarcode("1234567890",
                                                (int)MPos1dCodeType.MPOS_BARCODE_CODE128,
                                                2,  // Width (Dot)
                                                50, // Height (Dot)
                                                (int)MPosAlignment.MPOS_ALIGNMENT_CENTER,
                                                (int)MPosBarcodeTextPosition.MPOS_BARCODE_TEXT_BELOW);

                // Feed to tear-off position (Manual Cutter Position)
                await _printer.directIO(new byte[] { 0x1b, 0x4a, 0xaf });
            }
            catch (Exception ex){
                await DisplayAlert("Exception", ex.ToString(), "OK");
            }
            finally{
                // Printer starts printing by calling "setTransaction" function with "MPOS_PRINTER_TRANSACTION_OUT"
                await _printer.setTransaction((int)MPosTransactionMode.MPOS_PRINTER_TRANSACTION_OUT);
                // If there's nothing to do with the printer, call "closeService" method to disconnect the communication between Host and Printer.
                _printSemaphore.Release();
            }
        }

        async void OnPrint2dCodeClicked(object sender, EventArgs e)
        {
            // Prepares to communicate with the printer 
            _printer = await OpenPrinterService(_connectionInfo);

            if (_printer == null)
                return;

            try
            {
                await _printSemaphore.WaitAsync();

                // note : Page mode and transaction mode cannot be used together between IN and OUT.
                // When "setTransaction" function called with "MPOS_PRINTER_TRANSACTION_IN", print data are stored in the buffer.
                await _printer.setTransaction((int)MPosTransactionMode.MPOS_PRINTER_TRANSACTION_IN);
                // Printer Setting Initialize
                await _printer.directIO(new byte[] { 0x1b, 0x40 });
                await _printer.printQRCode("Hola Xamarin",
                                            (int)MPos2dCodeType.MPOS_BARCODE_QRCODE_MODEL1,
                                            (int)MPosAlignment.MPOS_ALIGNMENT_LEFT,
                                            3, // Size
                                            (int)MPosQRCodeECCLevel.MPOS_QRCODE_ECC_LEVEL_H);

                await _printer.printPDF417("Hola Xamarin",
                                (int)MPos2dCodeType.MPOS_BARCODE_PDF417,
                                (int)MPosAlignment.MPOS_ALIGNMENT_LEFT,
                                0, // number of columns
                                0, // number of rows
                                3, // module Width
                                6, // module Height
                                (int)MPosPDF417ECCLevel.MPOS_PDF417_ECC_LEVEL_0);

                // Feed to tear-off position (Manual Cutter Position)
                await _printer.directIO(new byte[] { 0x1b, 0x4a, 0xaf });
            }
            catch (Exception ex){
                await DisplayAlert("Exception", ex.ToString(), "OK");
            }
            finally{
                // Printer starts printing by calling "setTransaction" function with "MPOS_PRINTER_TRANSACTION_OUT"
                await _printer.setTransaction((int)MPosTransactionMode.MPOS_PRINTER_TRANSACTION_OUT);
                // If there's nothing to do with the printer, call "closeService" method to disconnect the communication between Host and Printer.
                _printSemaphore.Release();
            }
        }

        async void OnPrintImageClicked(object sender, EventArgs e)
        {
            // Prepares to communicate with the printer 
            _printer = await OpenPrinterService(_connectionInfo);

            if (_printer == null)
                return;

            try
            {
                await _printSemaphore.WaitAsync();

                // note : Page mode and transaction mode cannot be used together between IN and OUT.
                // When "setTransaction" function called with "MPOS_PRINTER_TRANSACTION_IN", print data are stored in the buffer.
                await _printer.setTransaction((int)MPosTransactionMode.MPOS_PRINTER_TRANSACTION_IN);
                // Printer Setting Initialize
                await _printer.directIO(new byte[] { 0x1b, 0x40 });

                string filePath = "";

                switch (Device.RuntimePlatform)
                {
                    case Device.iOS:
                        // From Project Resource
                        break;
                    case Device.Android:
                        // In Android, the image file should be in the SD card, and used as a first parameter to "printBitmapFile" method. 
                        IPlatformPathInfo platformPathInfo = DependencyService.Get<IPlatformPathInfo>();
                        filePath = platformPathInfo.GetAbsolutePath() + "/";
                        break;
                    case Device.Windows:
                    case Device.WinPhone:
                        // From Project Asset Directory 
                        filePath = "ms-appx:///Assets/";
                        break;
                }

                filePath += "bixolon_logo.bmp";

                await _printer.printBitmapFile(filePath,
                                            (int)MPosImageWidth.MPOS_IMAGE_WIDTH_ASIS,  // Image Width
                                            (int)MPosAlignment.MPOS_ALIGNMENT_CENTER,   // Alignment
                                            50,                                         // brightness
                                            true,                                       // Image Dithering
                                            true);                                      // Image Compress

                await _printer.printText("Bienvenido a Xamarin", new MPosFontAttribute() { Alignment = MPosAlignment.MPOS_ALIGNMENT_CENTER });
                // Feed to tear-off position (Manual Cutter Position)
                await _printer.directIO(new byte[] { 0x1b, 0x4a, 0xaf });
            }
            catch (Exception ex){
                await DisplayAlert("Exception", ex.ToString(), "OK");
            }
            finally{
                // Printer starts printing by calling "setTransaction" function with "MPOS_PRINTER_TRANSACTION_OUT"
                await _printer.setTransaction((int)MPosTransactionMode.MPOS_PRINTER_TRANSACTION_OUT);
                // If there's nothing to do with the printer, call "closeService" method to disconnect the communication between Host and Printer.
                _printSemaphore.Release();
            }
        }

        async void OnCheckPrinterStatusClicked(object sender, EventArgs e)
        {
            // Prepares to communicate with the printer 
            _printer = await OpenPrinterService(_connectionInfo);

            if (_printer == null)
                return;

            try
            {
                await _printSemaphore.WaitAsync();

                _printer.statusUpdateEvent += PrinterStatusEvent;

                var result = await _printer.checkPrinterStatus();

                if (result == (int)MPosPrinterStatus.MPOS_STATUS_NORMAL)
                    await DisplayAlert("Information", "Normal Status", "OK");

                // If there's nothing to do with the printer, call "closeService" method to disconnect the communication between Host and Printer.
            }
            catch (Exception ex){
                await DisplayAlert("Exception", ex.ToString(), "OK");
            }
            finally{
                _printSemaphore.Release();
            }
        }


        async void OnDeviceSelectionClicked(object sender, EventArgs e)
        {
            var page = new PrinterSelectionPage();
            page.BindingContext = this;
            await Navigation.PushAsync(page);
        }
    }
}
