﻿using System;
using Xamarin.Forms;
using Xamarin.Forms.Xaml;
using Plugin.BxlMpXamarinSDK;
using Plugin.BxlMpXamarinSDK.Abstractions;
using System.Threading.Tasks;
using System.Threading;

namespace XamarinMpSDKDemo
{
    [XamlCompilation(XamlCompilationOptions.Compile)]
    public partial class PageModePage : ContentPage
    {
        private MPosControllerPrinter _printer;
        private MposConnectionInformation _connectionInfo;
        private static SemaphoreSlim _printSemaphore = new SemaphoreSlim(1, 1);

        public PageModePage()
        {
            InitializeComponent();
            _printer        = null;
            _connectionInfo = null;
        }
        protected override void OnDisappearing()
        {
            if (_printer != null){
                OnDeviceCloseClicked(this, null);
            }
        }
        protected override void OnBindingContextChanged()
        {
            if (BindingContext == null)
                return;

            var connectionInfo = BindingContext as MposConnectionInformation;

            if (connectionInfo != null)
            {
                _connectionInfo = connectionInfo;

                switch (_connectionInfo.IntefaceType)
                {
                    case MPosInterfaceType.MPOS_INTERFACE_BLUETOOTH:
                    case MPosInterfaceType.MPOS_INTERFACE_WIFI:
                    case MPosInterfaceType.MPOS_INTERFACE_ETHERNET:
                        lblAddress.Text = connectionInfo.Address;
                        btnOpenService.IsEnabled = true;
                        break;
                    default:
                        return;
                }
            }
        }

        void PrinterStatusEvent(uint status)
        {
            // Any view interaction or any UI interaction should always be done in the main thread.
            Device.BeginInvokeOnMainThread(() =>
            {
                lblStatus.Text = "";

                if (status == (uint)MPosPrinterStatus.MPOS_STATUS_NORMAL)
                {
                    lblStatus.Text = "NORMAL ";
                    return;
                }
                if ((status & (uint)MPosPrinterStatus.MPOS_STATUS_PRINTER_ERROR) != 0)
                    lblStatus.Text += " PRINTER ERROR ";
                if ((status & (uint)MPosPrinterStatus.MPOS_STATUS_PRINTER_PAPER_NEAREND) != 0)
                    lblStatus.Text += " PAPER NEAR END ";
                if ((status & (uint)MPosPrinterStatus.MPOS_STATUS_PRINTER_COVEROPEN) != 0)
                    lblStatus.Text += " COVER OPEN ";
                if ((status & (uint)MPosPrinterStatus.MPOS_STATUS_PRINTER_PAPEREMPTY) != 0)
                    lblStatus.Text += " PAPER EMPTY ";
            });
        }

        async Task<MPosControllerPrinter> OpenPrinterService(MposConnectionInformation connectionInfo)
        {
            if (connectionInfo == null)
                return null;

            if (_printer != null)
                return _printer;

            btnOpenService.IsEnabled = false;

            _printer = MPosDeviceFactory.Current.createDevice(MPosDeviceType.MPOS_DEVICE_PRINTER) as MPosControllerPrinter;

            switch (connectionInfo.IntefaceType)
            {
                case MPosInterfaceType.MPOS_INTERFACE_BLUETOOTH:
                case MPosInterfaceType.MPOS_INTERFACE_WIFI:
                case MPosInterfaceType.MPOS_INTERFACE_ETHERNET:
                    _printer.selectInterface((int)connectionInfo.IntefaceType, connectionInfo.Address);
                    _printer.selectCommandMode((int)MPosCommandMode.MPOS_COMMAND_MODE_BYPASS);
                    break;
                default:
                    await DisplayAlert("Connection Fail", "Not Supported Interface", "OK");
                    return null;
            }

            await _printSemaphore.WaitAsync();

            try
            {
                var result = await _printer.openService();
                if (result != (int)MPosResult.MPOS_SUCCESS)
                {
                    _printer = null;
                    await DisplayAlert("Connection Fail", "openService failed. (" + result.ToString() + ")", "OK");
                }
            }
            finally
            {
                _printSemaphore.Release();
            }

            return _printer;
        }

        async void OnDeviceOpenClicked(object sender, EventArgs e)
        {
            if (_printer == null)
            {
                // Prepares to communicate with the printer 
                _printer = await OpenPrinterService(_connectionInfo);

                if (_printer != null)
                {
                    btnDirection.IsEnabled      = true;
                    btnPrintSample.IsEnabled    = true;
                    btnPrintStatus.IsEnabled    = true;
                    btnCloseService.IsEnabled   = true;
                    btnOpenService.IsEnabled    = false;
                }
                else
                {
                    btnDirection.IsEnabled      = false;
                    btnPrintSample.IsEnabled    = false;
                    btnPrintStatus.IsEnabled    = false;
                    btnCloseService.IsEnabled   = false;
                    btnOpenService.IsEnabled    = true;
                }
            }
        }

        async void OnDeviceCloseClicked(object sender, EventArgs e)
        {
            await _printSemaphore.WaitAsync();

            if (_printer != null){
                // If there's nothing to do with the printer, call "closeService" method to disconnect the communication between Host and Printer.
                await _printer.closeService();
                _printer = null;
                btnDirection.IsEnabled = false;
                btnPrintSample.IsEnabled = false;
                btnPrintStatus.IsEnabled = false;
                btnCloseService.IsEnabled = false;
                btnOpenService.IsEnabled = true;
            }

            _printSemaphore.Release();
        }

        async void OnPrintDirectionClicked(object sender, EventArgs e)
        {
            // Prepares to communicate with the printer 
            _printer = await OpenPrinterService(_connectionInfo);

            if (_printer == null)
                return;

            try
            {
                await _printSemaphore.WaitAsync();

                // Printer Setting Initialize
                await _printer.directIO(new byte[] { 0x1b, 0x40 });

                // note : Page mode and transaction mode cannot be used together.  
                // When "setPagemode" function called with "MPOS_PRINTER_PAGEMODE_IN", print data are stored in the buffer.
                await _printer.setPagemode((int)MPosPageMode.MPOS_PRINTER_PAGEMODE_IN);

                // Set printing area in page mode
                /* Note : MAX WIDTH
                 *   2 inch Printer = 48mm (384dots)
                 *   3 inch Printer = 72mm (576dots)
                 *   4 inch Printer = 104mm (832dots)
                 * Note : MAX HEIGHT
                 *   300mm (2400dots) regardless of the printers, 
                 */
                _printer.setPagemodePrintArea(0, 0, 576, 600);
                // Select print direction in page mode
                _printer.setPagemodeDirection((int)MPosPageModeDirection.MPOS_PRINTER_PD_0);        // Left to right (Upper Left)
                await _printer.printText("Hola Xamarin");
                _printer.setPagemodeDirection((int)MPosPageModeDirection.MPOS_PRINTER_PD_LEFT90);   // Bottom to Top (Lower Left)
                await _printer.printText("Hola Xamarin");
                _printer.setPagemodeDirection((int)MPosPageModeDirection.MPOS_PRINTER_PD_180);      // Right to Left (Lower Right)
                await _printer.printText("Hola Xamarin");
                _printer.setPagemodeDirection((int)MPosPageModeDirection.MPOS_PRINTER_PD_RIGHT90);  // Top to Bottom (Lower Left)
                await _printer.printText("Hola Xamarin");
            }
            catch (Exception ex){
                await DisplayAlert("Exception", ex.ToString(), "OK");
            }
            finally{
                // Printer starts printing by calling "setPagemode" function with "MPOS_PRINTER_PAGEMODE_OUT"
                await _printer.setPagemode((int)MPosPageMode.MPOS_PRINTER_PAGEMODE_OUT);
                // If there's nothing to do with the printer, call "closeService" method to disconnect the communication between Host and Printer.
                _printSemaphore.Release();
            }
        }

        async void OnPrintSampleClicked(object sender, EventArgs e)
        {
            // Prepares to communicate with the printer 
            _printer = await OpenPrinterService(_connectionInfo);

            if (_printer == null)
                return;

            try
            {
                await _printSemaphore.WaitAsync();

                int xPosition = 60;
                int yPosition = 20;

                // note : Page mode and transaction mode cannot be used together between IN and OUT.
                // When "setPagemode" function called with "MPOS_PRINTER_PAGEMODE_IN", print data are stored in the buffer.
                await _printer.setPagemode((int)MPosPageMode.MPOS_PRINTER_PAGEMODE_IN);

                // Set printing area in page mode
                /* Note : MAX WIDTH
                 *   2 inch Printer = 48mm (384dots)
                 *   3 inch Printer = 72mm (576dots)
                 *   4 inch Printer = 104mm (832dots)
                 * Note : MAX HEIGHT
                 *   300mm (2400dots) regardless of the printers, 
                 */
                _printer.setPagemodePrintArea(0, 0, 576, 600);
                // Select print direction in page mode
                _printer.setPagemodeDirection((int)MPosPageModeDirection.MPOS_PRINTER_PD_RIGHT90);  // Top to Bottom (Upper Right)

                // Prints file on this project. Image compress option may not work depending on the firmware version in page mode printing. So set that option with "false"
                _printer.setPagemodePosition(xPosition, yPosition);

                string filePath = "";

                switch (Device.RuntimePlatform)
                {
                    case Device.iOS:
                        // From Project Resource
                        break;
                    case Device.Android:
                        // In Android, the image file should be in the SD card, and used as a first parameter to "printBitmapFile" method. 
                        IPlatformPathInfo platformPathInfo = DependencyService.Get<IPlatformPathInfo>();
                        filePath = platformPathInfo.GetAbsolutePath() + "/";
                        break;
                    case Device.Windows:
                    case Device.WinPhone:
                        // From Project Asset Directory 
                        filePath = "ms-appx:///Assets/";
                        break;
                }

                filePath += "bixolon_logo.bmp";

                await _printer.printBitmapFile(filePath,
                                                (int)MPosImageWidth.MPOS_IMAGE_WIDTH_ASIS,      // Image Width
                                                (int)MPosAlignment.MPOS_ALIGNMENT_DEFAULT,      // Alignment
                                                50,                                             // brightness
                                                true,                                           // Image Dithering
                                                false);                                         // Image Compress

                _printer.setPagemodePosition(xPosition, (yPosition += 20));
                await _printer.printText("a Better Solution for your business\n");

                _printer.setPagemodePosition(xPosition, (yPosition += 60));
                await _printer.printQRCode("http://www.bixolon.com",
                                                (int)MPos2dCodeType.MPOS_BARCODE_QRCODE_MODEL1,
                                                (int)MPosAlignment.MPOS_ALIGNMENT_DEFAULT,
                                                3, // Size
                                                (int)MPosQRCodeECCLevel.MPOS_QRCODE_ECC_LEVEL_H);

                _printer.setPagemodePosition(xPosition, (yPosition += 10));
                await _printer.print1DBarcode("1234567890",
                                                (int)MPos1dCodeType.MPOS_BARCODE_CODE128,
                                                2,  // Width (Dot)
                                                50, // Height (Dot)
                                                (int)MPosAlignment.MPOS_ALIGNMENT_DEFAULT,
                                                (int)MPosBarcodeTextPosition.MPOS_BARCODE_TEXT_BELOW);

                _printer.setPagemodePosition(xPosition, (yPosition += 70));
                await _printer.printText("Hola Xamarin-1\n");

                _printer.setPagemodePosition(xPosition, (yPosition += 20));
                await _printer.printText("Hola Xamarin-2\n");

                _printer.setPagemodePosition(xPosition, (yPosition += 20));
                await _printer.printText("Hola Xamarin-3\n");
            }
            catch (Exception ex){
                await DisplayAlert("Exception", ex.ToString(), "OK");
            }
            finally{
                // Printer starts printing by calling "setPagemode" function with "MPOS_PRINTER_PAGEMODE_OUT"
                await _printer.setPagemode((int)MPosPageMode.MPOS_PRINTER_PAGEMODE_OUT);
                // If there's nothing to do with the printer, call "closeService" method to disconnect the communication between Host and Printer.
                _printSemaphore.Release();
            }
        }

        async void OnCheckPrinterStatusClicked(object sender, EventArgs e)
        {
            // Prepares to communicate with the printer 
            _printer = await OpenPrinterService(_connectionInfo);

            if (_printer == null)
                return;

            try
            {
                await _printSemaphore.WaitAsync();

                _printer.statusUpdateEvent += PrinterStatusEvent;

                var result = await _printer.checkPrinterStatus();

                if (result == (int)MPosPrinterStatus.MPOS_STATUS_NORMAL)
                    await DisplayAlert("Information", "Normal Status", "OK");

                // If there's nothing to do with the printer, call "closeService" method to disconnect the communication between Host and Printer.
            }
            catch (Exception ex){
                await DisplayAlert("Exception", ex.ToString(), "OK");
            }
            finally{
                _printSemaphore.Release();
            }
        }
        async void OnDeviceSelectionClicked(object sender, EventArgs e)
        {
            var page = new PrinterSelectionPage();
            page.BindingContext = this;
            await Navigation.PushAsync(page);
        }
    }
}
